/*
Benchmark for Route Planning Algorithms
 
Copyright (c) Microsoft Corporation

All rights reserved. 

MIT License
 
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#include <random>
#include <iostream>
#include <fstream>
#include <string>
#include <iomanip>

using namespace std;

int main(int argn, char **argv) {
	// Read input parameters.
	int32_t num(1000);
	if (argn > 1)
		num = stoi(argv[1]);
	int32_t seed(31101982);
	if (argn > 2)
		seed = stoi(argv[2]);
		
	cout << "Numbers: " << num << endl
		<< "Seed: " << seed << endl;
	

	ofstream file("random.h");
	if (!file.is_open()) return -1;
	
	// Instantiate Mersenne twister and uniform int distribution.
	cout << "Generating random numbers... " << flush;
	mt19937 twister(seed);
	uniform_int_distribution<int32_t> dist(0, numeric_limits<int32_t>::max());
	
	file << "// Calibration Benchmark for Survey Article" << endl
		<< "// \"Route Planning in Transportation Networks\"" << endl
		<< "// Thomas Pajor, tpajor@microsoft.com, 2014" << endl 
		<< "#pragma once" << endl
		<< endl
		<< "namespace benchmark {" << endl
		<< "\tconst int num_random_numbers = " << num << ";" << endl
		<< "\tconst int random_numbers[] = {";
	for (uint32_t i = 0; i < num; ++i) {
		if (i > 0) file << ",";
		const int32_t rnd = dist(twister);
		file << "0x" << hex << rnd;
	}
	file << "};" << endl
		<< "}" << endl;
	
	cout << "done." << endl;
	
	file.close();
	return 0;
}