/*
Benchmark for Route Planning Algorithms
 
Copyright (c) Microsoft Corporation

All rights reserved. 

MIT License
 
Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/
#pragma once

#include <vector>
#include <string>
#include <sstream>
#include <cstdlib>
using namespace std;

namespace benchmark {

// Returns a timestamp in milliseconds (incl. a fractional part).
#if defined(WIN32) || defined(WIN64) || defined(_WIN64) || defined(_WIN32) || defined(__WIN32__) || defined(__WIN64__) || defined(__CYGWIN__) || defined(_MSC_VER)

#include <windows.h>
inline double timestamp() {
	LARGE_INTEGER frequency;
	QueryPerformanceFrequency(&frequency);
	LARGE_INTEGER now;
	QueryPerformanceCounter(&now);
	return static_cast<double>(now.QuadPart) / static_cast<double>(frequency.QuadPart) * 1000.0;
}
#undef max
#undef min

#else // Linux / POSIX system

#include <sys/time.h>
inline double timestamp() {
	timeval tp;
	gettimeofday(&tp, NULL);
	double ms = double(tp.tv_usec) / 1000.;
	return tp.tv_sec*1000 + ms;
}

#endif

// Computes the median.
template<typename value_t>
inline value_t median(vector<value_t> vec) {
	if (vec.empty()) return value_t(0); // undefined.
	
	// Sort elements.
	sort(vec.begin(), vec.end());
	
	if (vec.size() % 2 == 0) {
		return (vec[vec.size()/2] + vec[vec.size()/2+1]) / 2.0;
	} else {
		return vec[vec.size()/2]; // middle element.
	}
}

// Splits a string into tokens using a delimiter character.
// Returns a vector of strings.
vector<string> split(const string s, const char delim) {
    stringstream ss(s);
    string item;
    vector<string> elems;
    while(getline(ss, item, delim)) {
    	elems.push_back(item);
    }
    return elems;
}

// Conversion between types.
template<typename out_t, typename in_t>
inline out_t convert(const in_t &in) {
	stringstream ss;
	out_t out = out_t();
	ss << in;
	ss >> out;
	return out; 
}

template<>
inline int convert(const string &s) {
	return atoi(s.c_str());
}

}
